export declare type TypedPathKey = string | symbol | number;
export declare type TypedPathFunction<ResultType> = (...args: any[]) => ResultType;
export declare type TypedPathHandlersConfig = Record<string, <T extends TypedPathHandlersConfig>(path: TypedPathKey[], additionalHandlers?: T) => any>;
declare const defaultHandlersConfig: {
    $path: (path: TypedPathKey[]) => string;
    /**
     * @deprecated This method transforms all path chunks to strings.
     * If you need the path with numbers and Symbols - use $rawPath
     */
    $raw: (path: TypedPathKey[]) => string[];
    $rawPath: (path: TypedPathKey[]) => TypedPathKey[];
    toString: (path: TypedPathKey[]) => () => string;
    [Symbol.toStringTag]: (path: TypedPathKey[]) => string;
    valueOf: (path: TypedPathKey[]) => () => string;
};
export declare type DefaultHandlers = typeof defaultHandlersConfig;
export declare type TypedPathHandlers<ConfigType extends TypedPathHandlersConfig> = {
    [key in keyof ConfigType]: ReturnType<ConfigType[key]>;
};
export declare type TypedPathWrapper<OriginalType, HandlersType extends TypedPathHandlers<Record<never, never>>> = (OriginalType extends Array<infer OriginalArrayItemType> ? {
    [index: number]: TypedPathWrapper<OriginalArrayItemType, HandlersType>;
} : OriginalType extends TypedPathFunction<infer OriginalFunctionResultType> ? {
    (): TypedPathWrapper<OriginalFunctionResultType, HandlersType>;
} & {
    [P in keyof Required<OriginalFunctionResultType>]: TypedPathWrapper<OriginalFunctionResultType[P], HandlersType>;
} : {
    [P in keyof Required<OriginalType>]: TypedPathWrapper<OriginalType[P], HandlersType>;
}) & TypedPathHandlers<HandlersType>;
export declare function typedPath<OriginalObjectType, HandlersType extends TypedPathHandlersConfig = Record<never, never>>(additionalHandlers?: HandlersType, path?: TypedPathKey[]): TypedPathWrapper<OriginalObjectType, HandlersType & DefaultHandlers>;
export {};
